import ssl
from grpclib.client import Channel as gRPClibChannel
from tempfile import mkstemp
from .device_ca_cert import ca_cert
from grpclib.events import SendRequest, listen
import inspect


class Channel(gRPClibChannel):
    """
    Channel class is used to securely connect to a Qb2 device.

    The connection is encrypted and authenticated.
    The serial_number can be passed additionally to authenticate a particular device.
    If it is not supplied, the authentication only checks if it is a Blickfeld device.

    ::param fqdn_or_ip: Fully qualified domain name e.g. qb2-xxxxx.blickfeld.com, hostname, or IP address
    :type fqdn_or_ip: str
    param serial_number: Unique serial number assigned to each Qb2 device (corresponding serial number is written on the device label)
    :type serial_number: str
    :param port: Port on which the Qb2 device is reachable (the default is 55551)
    :type port: uint
    :param device_ca_cert: String containing a root certificate for SSL connection
    :type device_ca_cert: str
    :param token: String containing an access token or an awaitable which is called on every method call
    """

    def __init__(
        self, fqdn_or_ip: str, serial_number: str = None, port=55551, device_ca_cert=ca_cert, token=None, metadata=None
    ) -> None:
        # Store all properties to allow clone
        self.fqdn_or_ip = fqdn_or_ip
        self.serial_number = serial_number
        self.port = port
        self.device_ca_cert = device_ca_cert
        self.token = token
        self.metadata = metadata

        # save device ca certificate key to temporal file
        fd, key_and_cert_file = mkstemp()
        with open(fd, "w") as f:
            f.write(device_ca_cert)

        # create SSL context for client
        ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)  # client side context
        ctx.verify_mode = ssl.CERT_REQUIRED  # TLS
        ctx.check_hostname = False  # disable hostname check
        ctx.set_alpn_protocols(["h2"])
        try:
            ctx.load_verify_locations(cafile=key_and_cert_file)
        except ssl.SSLError as error:
            raise RuntimeError("Device CA certificate is of incorrect format:" + error.reason)

        # TODO Check serial number. Authentication of device serial number is not yet implemented.

        # establish gRPC secured channel
        super().__init__(host=fqdn_or_ip, port=port, ssl=ctx)
        # if token is provided -> inject it to every outgoing send request
        if token:
            listen(self, SendRequest, self.__inject_token)

        if metadata:
            listen(self, SendRequest, self.__inject_metadata)

    def clone(self, token=None):
        return Channel(
            fqdn_or_ip=self.fqdn_or_ip,
            serial_number=self.serial_number,
            port=self.port,
            device_ca_cert=self.device_ca_cert,
            token=token if token != self.token else self.token,
            metadata=self.metadata,
        )

    async def __inject_token(self, event: SendRequest):
        # if token is method, call it to get token
        token = self.token
        if callable(token):
            token = token(self)
        if inspect.isawaitable(token):
            token = await token
        event.metadata["token"] = token

    async def __inject_metadata(self, event: SendRequest):
        event.metadata.update(self.metadata)

    async def __aenter__(self):
        return self

    async def __aexit__(self, type, value, traceback):
        self.close()

    # Allow synchronous usage
    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.close()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/hardware/services/compute_module.proto, blickfeld/hardware/services/identification.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import config as _config__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class IdentificationGetResponse(betterproto.Message):
    """Response message for the identification get command"""

    identification: "_config__.Identification" = betterproto.message_field(1)
    """The current identification"""


class ComputeModule(betterproto.ServiceStub):
    """
    This service is used to control & observe the state of the compute module.
    """

    async def async_reboot(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Reboots the CM4"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.hardware.services.ComputeModule/Reboot",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def reboot(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Reboots the CM4"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_reboot(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Identification(betterproto.ServiceStub):
    """
    The identification service provides methods to set and read out
    identification data of the Qb2 device.
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "IdentificationGetResponse":
        """Gets identification configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.hardware.services.Identification/Get",
            request,
            IdentificationGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "IdentificationGetResponse":
        """Gets identification configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/base/options/access_control.proto, blickfeld/base/options/misc.proto, blickfeld/base/options/protocol_maturity.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class AccessControlLevel(betterproto.Enum):
    """
    Available access levels The levels are ordered ascending, which means that
    each level also inherits the permissions of the levels above it.
    """

    LEVEL_UNSPECIFIED = 0
    """Access level is not specified"""

    LEVEL_PUBLIC = 1
    """
    Everyone can access The corresponding entities can be accessed by anyone
    who has access to the interface of the device. The access can not be
    disabled by the user.
    """

    LEVEL_AUTHORIZED = 2
    """
    Only authorized clients can access The corresponding entities can only be
    used by authorized clients. The access can be managed by clients with the
    ADMIN access level. If desired, the AUTHORIZED access level can be
    configured to act equally to the PUBLIC access level.
    """

    LEVEL_ADMIN = 3
    """
    Only administrative clients can access The corresponding entities are
    mainly used to configure the access tokens of the AUTHORIZED access level.
    It may also be used to manage configuration which might void the warranty
    or specification of the device.
    """

    LEVEL_SUPPORT = 4
    """
    Only clients of the Blickfeld support can access The corresponding entities
    can only be accessed by Blickfeld Support Tools & clients. The access
    tokens are bound to devices.
    """

    LEVEL_PRODUCTION = 5
    """
    Only clients in production can access The corresponding entities are used
    for end-of-line & re-work processes.
    """

    LEVEL_DEVELOPER = 6
    """Only Blickfeld developers can access"""


class ProtocolMaturityLevel(betterproto.Enum):
    """
    This enumeration describes the maturity of a protocol. Please refer to the
    individual values. [IMPORTANT] APIs marked with the EXPERIMENTAL &
    DEPRECATED flags have to be handled with care. Please read through the
    definition. Please report if the protocol maturity of a service, method,
    and field does not fit e.g. if an EXPERIMENTAL field is required in an
    application & production use case.
    """

    LEVEL_UNSPECIFIED = 0
    """
    Level is not set. Default level for services is EXPERIMENTAL. Default level
    for methods & fields is INHERITED.
    """

    LEVEL_EXPERIMENTAL = 1
    """
    APIs marked with this flag should be handled with care. They might not
    function as expected. Additionally it is not ensure that the APIs will ever
    be moved to the MAINTAINED level. They might be altered or removed in any
    upcoming release without further notice.
    """

    LEVEL_MAINTAINED = 2
    """
    APIs marked with this flag are actively maintained and released. They will
    not change their behaviour without notices in the changelog. Breaking
    changes in a maintained APIs have to result in a major update of the
    module.
    """

    LEVEL_DEPRECATED = 3
    """
    APIs marked with this flag are no longer actively maintained. They will be
    removed in upcoming releases.
    """

    LEVEL_INHERITED = 4
    """The protocol maturity is inherited from the parent"""


@dataclass(eq=False, repr=False)
class AccessControl(betterproto.Message):
    """
    Access Control message which specifies the required access level and
    licenses for protocol entities. Protocol entities are usually gRPC services
    but can also be more fine grained on gRPC method and Protobuf field level.
    """

    level: "AccessControlLevel" = betterproto.enum_field(1)
    """Minimum access level required for entity"""


@dataclass(eq=False, repr=False)
class LinkData(betterproto.Message):
    type: str = betterproto.string_field(1)
    directory: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ProtocolMaturity(betterproto.Message):
    """
    This message specifies the maturity of a protocol for customer- &
    production-facing APIs.
    """

    level: "ProtocolMaturityLevel" = betterproto.enum_field(1)
    """Simple level which describes maturity"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/detector/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    Health message giving information about operation status and possible
    errors.
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of the module."""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is OK."""


@dataclass(eq=False, repr=False)
class HealthApd(betterproto.Message):
    """Current status of APD operation."""

    bias_voltage: float = betterproto.float_field(1)
    """Current APD bias voltage."""

    temperature_compensation: bool = betterproto.bool_field(2)
    """
    Flag indicating whether temperature compensation is used. If not, static
    voltage is used.
    """

    multiplication_factor: int = betterproto.uint32_field(3)
    """
    Currently used multiplication factor. Only valid if temperature
    compensation is used.
    """

    temperature: float = betterproto.float_field(4)
    """
    Current temperature measured with dedicated temperature IC which is
    thermally coupled to the APD.
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_pipeline/services/data_source.proto, blickfeld/percept_pipeline/services/health.proto, blickfeld/percept_pipeline/services/perception.proto, blickfeld/percept_pipeline/services/runtime_action.proto, blickfeld/percept_pipeline/services/zone.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    List,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class DataSourceGetResponse(betterproto.Message):
    """Response containing the current data source"""

    data_source: "_config__.DataSource" = betterproto.message_field(1)
    """The currently set data source"""


@dataclass(eq=False, repr=False)
class DataSourceSetRequest(betterproto.Message):
    """Request for setting the data source"""

    data_source: "_config__.DataSource" = betterproto.message_field(1)
    """The data source to be set"""


@dataclass(eq=False, repr=False)
class DataSourceWatchResponse(betterproto.Message):
    """Continuous response on data source changes"""

    data_source: "_config__.DataSource" = betterproto.message_field(1)
    """The currently set data source"""


@dataclass(eq=False, repr=False)
class RuntimeActionGetTareVolumeRequest(betterproto.Message):
    """Request for the raw volume to be used for taring a volume zone"""

    zone_uuid: str = betterproto.string_field(1)
    """The uuid of the volume zone"""


@dataclass(eq=False, repr=False)
class RuntimeActionGetTareVolumeResponse(betterproto.Message):
    """Response containing the raw volume of the requested zone"""

    tare_volume: float = betterproto.float_field(1)
    """
    The raw volume of the volume zone without any offsets/modifications. The
    value is supposed to be configured as 'empty_volume' value of a volume
    zone. As a result the current volume of the zone will be '0' until more
    volume is added.
    """


@dataclass(eq=False, repr=False)
class RuntimeActionResetBackgroundRequest(betterproto.Message):
    """
    Request to reset the background model of one, multiple or all devices
    """

    fqdns: List[str] = betterproto.string_field(1)
    """
    If the list is empty, the background models of all data sources are reset.
    If the background models of only specific devices should be rebuilt, the
    'fqdn' of the respective devices has to be mentioned in the 'fqdns' list.
    """


@dataclass(eq=False, repr=False)
class PerceptionGetResponse(betterproto.Message):
    """Response containing the current perception configuration"""

    perception: "_config__.Perception" = betterproto.message_field(1)
    """The current perception configuration"""


@dataclass(eq=False, repr=False)
class PerceptionSetRequest(betterproto.Message):
    """Request for setting the perception configuration"""

    template_name: str = betterproto.string_field(1, group="set_type")
    """
    Use a template to set the configuration, templates are predefined
    perception configurations
    """

    perception: "_config__.Perception" = betterproto.message_field(2, group="set_type")
    """Use the specified configuration"""


@dataclass(eq=False, repr=False)
class PerceptionWatchResponse(betterproto.Message):
    """Continuous response on perception configuration changes"""

    perception: "_config__.Perception" = betterproto.message_field(1)
    """The current set perception configuration"""


@dataclass(eq=False, repr=False)
class PerceptionListTemplatesResponse(betterproto.Message):
    """Response containing perception configuration templates"""

    named_templates: Dict[str, "_config__.Perception"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    Mapping from a template name to the template itself. The template can be
    used to configure the perception algorithms for certain use-cases.
    """


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """Stream response containing health of the module"""

    health: "_data__.Health" = betterproto.message_field(1)
    """The module health"""


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """Get response containing health of the module"""

    health: "_data__.Health" = betterproto.message_field(1)
    """The module health"""


@dataclass(eq=False, repr=False)
class ZoneListResponse(betterproto.Message):
    """Response containing the configured list of zones"""

    zones: List["_config__.ZoneAlgorithm"] = betterproto.message_field(1)
    """The currently configured zones"""


@dataclass(eq=False, repr=False)
class ZoneStoreRequest(betterproto.Message):
    """Request to store or update an existing zone"""

    zone: "_config__.ZoneAlgorithm" = betterproto.message_field(1)
    """
    The new zone to be added. If there is a zone with the same uuid, that zone
    will get updated with this configuration.
    """


@dataclass(eq=False, repr=False)
class ZoneDeleteRequest(betterproto.Message):
    """Request to delete a zone"""

    uuid: str = betterproto.string_field(1)
    """The uuid of the zone to be deleted"""


@dataclass(eq=False, repr=False)
class ZoneWatchResponse(betterproto.Message):
    """Continuous response on zone list changes"""

    zones: List["_config__.ZoneAlgorithm"] = betterproto.message_field(1)
    """The current/updated list of zones"""


class DataSource(betterproto.ServiceStub):
    """
    The data source service allows getting, configuring and receiving updates
    of the data source used for processing pipelines. The data source
    configuration specifies where to retrieve the point cloud data for further
    processing.
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "DataSourceGetResponse":
        """Get current data_source Configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.DataSource/Get",
            request,
            DataSourceGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "DataSourceGetResponse":
        """Get current data_source Configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_set(
        self,
        *,
        data_source: "_config__.DataSource" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set configuration for data_source"""

        request = DataSourceSetRequest()
        if data_source is not None:
            request.data_source = data_source

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.DataSource/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        data_source: "_config__.DataSource" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set configuration for data_source"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                data_source=data_source,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["DataSourceWatchResponse"]:
        """Watch configuration for data_source"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_pipeline.services.DataSource/Watch",
            request,
            DataSourceWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["DataSourceWatchResponse"]:
        """Watch configuration for data_source"""

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class RuntimeAction(betterproto.ServiceStub):
    """
    The RuntimeAction service enables the system to execute runtime actions and
    updates on a running pipeline.
    """

    async def async_get_tare_volume(
        self,
        *,
        zone_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "RuntimeActionGetTareVolumeResponse":
        """Computes raw volume within a volume zone without any offsets."""

        request = RuntimeActionGetTareVolumeRequest()
        request.zone_uuid = zone_uuid

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.RuntimeAction/GetTareVolume",
            request,
            RuntimeActionGetTareVolumeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_tare_volume(
        self,
        *,
        zone_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "RuntimeActionGetTareVolumeResponse":
        """Computes raw volume within a volume zone without any offsets."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_tare_volume(
                zone_uuid=zone_uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_reset_background(
        self,
        *,
        fqdns: Optional[List[str]] = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Resets the background of one, multiple or all devices"""

        fqdns = fqdns or []

        request = RuntimeActionResetBackgroundRequest()
        request.fqdns = fqdns

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.RuntimeAction/ResetBackground",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def reset_background(
        self,
        *,
        fqdns: Optional[List[str]] = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Resets the background of one, multiple or all devices"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_reset_background(
                fqdns=fqdns,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Perception(betterproto.ServiceStub):
    """
    The perception service allows getting, configuring and receiving updates of
    the perception configurations. Perception configurations specify the
    algorithms used for e.g. for background subtraction etc.
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PerceptionGetResponse":
        """Get current perception configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.Perception/Get",
            request,
            PerceptionGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PerceptionGetResponse":
        """Get current perception configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_set(
        self,
        *,
        template_name: str = "",
        perception: "_config__.Perception" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set the perception configuration"""

        request = PerceptionSetRequest()
        request.template_name = template_name
        if perception is not None:
            request.perception = perception

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.Perception/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        template_name: str = "",
        perception: "_config__.Perception" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set the perception configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                template_name=template_name,
                perception=perception,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["PerceptionWatchResponse"]:
        """Watch changes of perception configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_pipeline.services.Perception/Watch",
            request,
            PerceptionWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["PerceptionWatchResponse"]:
        """Watch changes of perception configuration"""

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_list_templates(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PerceptionListTemplatesResponse":
        """Return list of names for named templates"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.Perception/ListTemplates",
            request,
            PerceptionListTemplatesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list_templates(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PerceptionListTemplatesResponse":
        """Return list of names for named templates"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list_templates(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Health(betterproto.ServiceStub):
    """Reports the health of the pipeline module."""

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        Streams health information on a regular cadence or if certain incidents
        happen
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_pipeline.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        Streams health information on a regular cadence or if certain incidents
        happen
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health information"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health information"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Zone(betterproto.ServiceStub):
    """
    The zone service allows getting, configuring, deleting and receiving
    updates of the configured zones.
    """

    async def async_list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "ZoneListResponse":
        """Returns the configured zones"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.Zone/List",
            request,
            ZoneListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "ZoneListResponse":
        """Returns the configured zones"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_store(
        self,
        *,
        zone: "_config__.ZoneAlgorithm" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store / Update a zone"""

        request = ZoneStoreRequest()
        if zone is not None:
            request.zone = zone

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.Zone/Store",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store(
        self,
        *,
        zone: "_config__.ZoneAlgorithm" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store / Update a zone"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store(
                zone=zone,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete a zone"""

        request = ZoneDeleteRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.percept_pipeline.services.Zone/Delete",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete a zone"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_delete(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["ZoneWatchResponse"]:
        """Watch configuration for zone list"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_pipeline.services.Zone/Watch",
            request,
            ZoneWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["ZoneWatchResponse"]:
        """Watch configuration for zone list"""

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_processing/data/data_type.proto, blickfeld/percept_processing/data/health.proto, blickfeld/percept_processing/data/state.proto, blickfeld/percept_processing/data/state_list.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import (
    data as __base_data__,
    geometry as __base_geometry__,
)
from ...percept_pipeline import data as __percept_pipeline_data__


class DataType(betterproto.Enum):
    """The type of data that the stream contains"""

    DATA_TYPE_UNSPECIFIED = 0
    """The zero value should not be used"""

    DATA_TYPE_HEALTH = 1
    """
    Health data type: this contains information about the health of the
    pipeline and of the module
    """

    DATA_TYPE_POINT_CLOUD = 2
    """Point cloud data type: a complete point cloud message is streamed"""

    DATA_TYPE_STATE_LIST = 4
    """
    State list data type: a list of states generated by the zones configured in
    the pipeline
    """


@dataclass(eq=False, repr=False)
class State(betterproto.Message):
    """
    A state transmits high-level information about the scene or setup. The
    information it contains depends on the configuration. States will get
    calculated on every frame of measurement.
    """

    timestamp: datetime = betterproto.message_field(1)
    """Timestamp of the lidar frame in which the state was measured"""

    generator_uuid: str = betterproto.string_field(2)
    """The uuid of the entity generating this state"""

    generator_name: str = betterproto.string_field(8)
    """The name of the entity generating this state"""

    security: "StateSecurity" = betterproto.message_field(5, group="state_type")
    """The security state contains intrusion information of a zone"""

    volume: "StateVolume" = betterproto.message_field(7, group="state_type")
    """Current volume measured in the selected zone"""


@dataclass(eq=False, repr=False)
class StateSecurity(betterproto.Message):
    """A message containing an alarm state of a zone."""

    alarm: bool = betterproto.bool_field(1)
    """If there is an alarm or not in the zone"""

    num_detected_points: int = betterproto.uint32_field(2)
    """The current number of lidar points detected in the zone"""


@dataclass(eq=False, repr=False)
class StateVolume(betterproto.Message):
    """
    A message containing the volume state of a zone. This is based on one
    volume zone.
    """

    volume: float = betterproto.float_field(1)
    """
    The indices of Volume Tiles are expressed w.r.t the Volume Map  coordinate
    system. The tile with index (0,0) will contain the points  from [0,
    tile_size), [0, tile_size). That means that (0,0,0) of the  Volume Map
    frame does not coincide with the center of the tile with  index (0,0) but
    with the crossing of the 4 tiles [(0,0), (0, -1,),  (-1,-1), (-1,0)].
    Additionally the volume tile could have multiple vertical clusters that
    will represent the data inside the tile. The current volume of the zone
    """

    tiles: List["StateVolumeVolumeTile"] = betterproto.message_field(2)
    """
    The tiles of the volume map The volume of a zone is approximated by a
    'volume map' see xref:blickfeld/percept_pipeline/config/zone_algorithm.adoc
    #_blickfeld_percept_pipeline_config_ZoneAlgorithm_Volume[Volume] for more
    details.
    """

    tile_size: float = betterproto.float_field(3)
    """The tile size of the volume map"""

    shape: "__base_geometry__.Shape" = betterproto.message_field(4)
    """
    The shape of the volume map, defining the pose and the dimension of the
    volume map in world frame. The shape is similar to the shape of the input
    zone with the following difference: - shape.pose.position:
    (x=zone.pose.position.x, y=zone.pose.position.y, z=0) -
    shape.pose.rotation: (roll=0, pitch=0, yaw=zone.pose.rotation.yaw) -
    shape.box.dimension: (x=zone.box.dimension.x, y=zone.box.dimension.y, z=0)
    """


@dataclass(eq=False, repr=False)
class StateVolumeVolumeTile(betterproto.Message):
    """Message of a single volume tile"""

    index_x: int = betterproto.int32_field(1)
    """Grid Map Tile Index X (can be negative and positive)"""

    index_y: int = betterproto.int32_field(2)
    """Grid Map Tile Index Y (can be negative and positive)"""

    heights: List[float] = betterproto.float_field(3)
    """Height per cluster, ordered ascending"""

    std_deviations: List[float] = betterproto.float_field(4)
    """Standard Deviation per cluster"""

    number_points: List[float] = betterproto.float_field(5)
    """Points per cluster"""


@dataclass(eq=False, repr=False)
class StateList(betterproto.Message):
    """
    A message that contains a list of states. A state is an information
    generated by the system based on the select zone type or device. For
    example, a state can contain information about the volume or the intrusion
    in the zone.
    """

    timestamp: datetime = betterproto.message_field(1)
    """The timestamp of the message construction."""

    states: List["State"] = betterproto.message_field(2)
    """List of states."""


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    A health message that contains information about the pipeline status and
    the module itself.
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module."""

    state_reason: str = betterproto.string_field(2)
    """Reason for the given state. It is not set if state is STATE_OK."""

    pipeline_state: "__percept_pipeline_data__.State" = betterproto.enum_field(4)
    """
    Current state of the pipeline. It indicates if the pipeline is stable,
    unstable or broken, based on the frequency of the data that is processed
    and published.
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/system/services/firmware.proto, blickfeld/system/services/network.proto, blickfeld/system/services/scan_pattern.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterable,
    AsyncIterator,
    Dict,
    Iterable,
    Iterator,
    Optional,
    Union,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class FirmwareUploadRequest(betterproto.Message):
    """
    This request is sent multiple times to upload a full complete bundle. End
    the stream with the last chunk of the bundle.
    """

    binary_chunk: bytes = betterproto.bytes_field(1)
    """
    Binary chunk of firmware bundle. The chunk size should not exceed 1 MB. For
    a firmware bundle with a size of 500 MB, at least 500 chunks / messages are
    required.
    """


@dataclass(eq=False, repr=False)
class FirmwareUploadResponse(betterproto.Message):
    """
    The uploaded firmware bundle is validated after the upload and its firmware
    info is returned.
    """

    info: "_data__.FirmwareInfo" = betterproto.message_field(1)
    """Firmware info with version and release information"""


@dataclass(eq=False, repr=False)
class FirmwareInstallRequest(betterproto.Message):
    """
    This request contains some optional configuration parameters for the
    install step.
    """

    pass


@dataclass(eq=False, repr=False)
class FirmwareInstallResponse(betterproto.Message):
    """Continous status message which reports the installation progress."""

    status: "_data__.FirmwareStatus" = betterproto.message_field(1)
    """Progress is defined in Status.Installation.Progress."""


@dataclass(eq=False, repr=False)
class FirmwareUploadAndInstallRequest(betterproto.Message):
    """
    Please refer to <<_blickfeld_system_services_FirmwareUploadRequest,
    FirmwareUploadRequest>>.
    """

    binary_chunk: bytes = betterproto.bytes_field(1)
    """
    Please refer to <<_blickfeld_system_services_FirmwareUploadRequest,
    FirmwareUploadRequest>>.
    """


@dataclass(eq=False, repr=False)
class FirmwareUploadAndInstallResponse(betterproto.Message):
    """
    Please refer to <<_blickfeld_system_services_FirmwareInstallResponse,
    FirmwareInstallResponse>>.
    """

    status: "_data__.FirmwareStatus" = betterproto.message_field(1)
    """
    Please refer to <<_blickfeld_system_services_FirmwareInstallResponse,
    FirmwareInstallResponse>>.
    """


@dataclass(eq=False, repr=False)
class FirmwareGetStatusResponse(betterproto.Message):
    """Response with snapshot of current status."""

    status: "_data__.FirmwareStatus" = betterproto.message_field(1)
    """Current firmware status"""


@dataclass(eq=False, repr=False)
class FirmwareWatchStatusResponse(betterproto.Message):
    """Continuous response with status"""

    status: "_data__.FirmwareStatus" = betterproto.message_field(1)
    """Current firmware status"""


@dataclass(eq=False, repr=False)
class NetworkValidateRequest(betterproto.Message):
    """Request to validate method"""

    config: "_config__.Network" = betterproto.message_field(1)
    """Configuration which should be validated"""


@dataclass(eq=False, repr=False)
class NetworkValidateResponse(betterproto.Message):
    """Response to validate method"""

    status: "_data__.NetworkStatus" = betterproto.message_field(1)
    """Status message with allocated IP addresses"""


@dataclass(eq=False, repr=False)
class NetworkGetMacAddressesResponse(betterproto.Message):
    """Response with mac addresses"""

    ethernet: str = betterproto.string_field(1)
    """Mac address of ethernet / LAN interface"""

    wireless: str = betterproto.string_field(2)
    """Mac address of wireless / WiFi interface"""


@dataclass(eq=False, repr=False)
class ScanPatternGetResponse(betterproto.Message):
    name: str = betterproto.string_field(1)
    scan_pattern: "_config__.ScanPattern" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ScanPatternSetRequest(betterproto.Message):
    name: str = betterproto.string_field(1, group="type_oneof")
    scan_pattern: "_config__.ScanPattern" = betterproto.message_field(
        2, group="type_oneof"
    )


@dataclass(eq=False, repr=False)
class ScanPatternListResponse(betterproto.Message):
    named_scan_patterns: Dict[str, "_config__.ScanPattern"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass(eq=False, repr=False)
class ScanPatternStoreRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    scan_pattern: "_config__.ScanPattern" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ScanPatternDeleteRequest(betterproto.Message):
    name: str = betterproto.string_field(1)


class Firmware(betterproto.ServiceStub):
    """
    The firmware service offers methods to fetch the currently installed
    firmware, upgrade it, downgrade it and also configure auto-update
    mechanisms.
    """

    async def async_upload(
        self,
        request_iterator: Union[
            AsyncIterable["FirmwareUploadRequest"], Iterable["FirmwareUploadRequest"]
        ],
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FirmwareUploadResponse":
        """
        Upload the firmware bundle from the client to the device. Fails if the
        uploaded firmware bundle is not valid.
        """

        return await self._stream_unary(
            "/blickfeld.system.services.Firmware/Upload",
            request_iterator,
            FirmwareUploadRequest,
            FirmwareUploadResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def upload(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FirmwareUploadResponse":
        """
        Upload the firmware bundle from the client to the device. Fails if the
        uploaded firmware bundle is not valid.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_upload(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_install(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["FirmwareInstallResponse"]:
        """
        Trigger the installation process of an uploaded firmware. A status
        stream with the installation progress is returned and ends with
        installation completion.
        """

        request = FirmwareInstallRequest()

        async for response in self._unary_stream(
            "/blickfeld.system.services.Firmware/Install",
            request,
            FirmwareInstallResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def install(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["FirmwareInstallResponse"]:
        """
        Trigger the installation process of an uploaded firmware. A status
        stream with the installation progress is returned and ends with
        installation completion.
        """

        loop = asyncio.get_event_loop()
        ait = self.async_install(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_upload_and_install(
        self,
        request_iterator: Union[
            AsyncIterable["FirmwareUploadAndInstallRequest"],
            Iterable["FirmwareUploadAndInstallRequest"],
        ],
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["FirmwareUploadAndInstallResponse"]:
        """Combines the Upload and Install steps in a single method."""

        async for response in self._stream_stream(
            "/blickfeld.system.services.Firmware/UploadAndInstall",
            request_iterator,
            FirmwareUploadAndInstallRequest,
            FirmwareUploadAndInstallResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def upload_and_install(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["FirmwareUploadAndInstallResponse"]:
        """Combines the Upload and Install steps in a single method."""

        loop = asyncio.get_event_loop()
        ait = self.async_upload_and_install(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get_status(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FirmwareGetStatusResponse":
        """
        Get the current firmware status which includes the currently installed
        firmware but also ongoing or failed installations.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.system.services.Firmware/GetStatus",
            request,
            FirmwareGetStatusResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_status(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FirmwareGetStatusResponse":
        """
        Get the current firmware status which includes the currently installed
        firmware but also ongoing or failed installations.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_status(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch_status(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["FirmwareWatchStatusResponse"]:
        """
        Continously watch the status stream and get updates on changes. This
        can be used to attach to an ongoing installation.
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.system.services.Firmware/WatchStatus",
            request,
            FirmwareWatchStatusResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch_status(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["FirmwareWatchStatusResponse"]:
        """
        Continously watch the status stream and get updates on changes. This
        can be used to attach to an ongoing installation.
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch_status(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Network(betterproto.ServiceStub):
    """
    The network service allows to setup network configuration of Qb2 device.
    The exact structure of configuration settings is defined by "Network"
    protocol message. Service provides "SetConfig()" and "GetConfig()" API
    which allow to apply and read out configuration settings at run-time. The
    "GetStatus()" API brings configuration and statistical information about
    currently active network connection.
    """

    async def async_get_status(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "_data__.NetworkStatus":
        """Brings information about currently active connection"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.system.services.Network/GetStatus",
            request,
            _data__.NetworkStatus,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_status(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "_data__.NetworkStatus":
        """Brings information about currently active connection"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_status(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_set_config(
        self,
        *,
        hostname: str = "",
        dhcp: bool = False,
        manual: "NetworkStatic" = None,
        link_local: bool = False,
        wireless: "NetworkWireless" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Applies specified network configuration"""

        request = _config__.Network()
        request.hostname = hostname
        request.dhcp = dhcp
        if manual is not None:
            request.manual = manual
        request.link_local = link_local
        if wireless is not None:
            request.wireless = wireless

        return await self._unary_unary(
            "/blickfeld.system.services.Network/SetConfig",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set_config(
        self,
        *,
        hostname: str = "",
        dhcp: bool = False,
        manual: "NetworkStatic" = None,
        link_local: bool = False,
        wireless: "NetworkWireless" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Applies specified network configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set_config(
                hostname=hostname,
                dhcp=dhcp,
                manual=manual,
                link_local=link_local,
                wireless=wireless,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get_config(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "_config__.Network":
        """Reads out provided network configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.system.services.Network/GetConfig",
            request,
            _config__.Network,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_config(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "_config__.Network":
        """Reads out provided network configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_config(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_validate(
        self,
        *,
        config: "_config__.Network" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "NetworkValidateResponse":
        """
        Validates the network configuration by applying it shortly. If DHCP is
        configured, it checks if it can acquire an IPv4 address. For static IP
        address, the validation only works for wireless networks. It then,
        still cannot allocate a manual IP address if the credentials are wrong.
        [NOTE] The validation does not yet separate error messages for
        misconfigured wireless credentials or wrongly configured DHCP server.
        Thus, the issue is not necessarily on the device side.
        """

        request = NetworkValidateRequest()
        if config is not None:
            request.config = config

        return await self._unary_unary(
            "/blickfeld.system.services.Network/Validate",
            request,
            NetworkValidateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def validate(
        self,
        *,
        config: "_config__.Network" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "NetworkValidateResponse":
        """
        Validates the network configuration by applying it shortly. If DHCP is
        configured, it checks if it can acquire an IPv4 address. For static IP
        address, the validation only works for wireless networks. It then,
        still cannot allocate a manual IP address if the credentials are wrong.
        [NOTE] The validation does not yet separate error messages for
        misconfigured wireless credentials or wrongly configured DHCP server.
        Thus, the issue is not necessarily on the device side.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_validate(
                config=config,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get_mac_addresses(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "NetworkGetMacAddressesResponse":
        """Get MAC addresses of the available interfaces"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.system.services.Network/GetMacAddresses",
            request,
            NetworkGetMacAddressesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_mac_addresses(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "NetworkGetMacAddressesResponse":
        """Get MAC addresses of the available interfaces"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_mac_addresses(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class ScanPattern(betterproto.ServiceStub):
    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "ScanPatternGetResponse":
        """Gets scan pattern configuration settings"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.system.services.ScanPattern/Get",
            request,
            ScanPatternGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "ScanPatternGetResponse":
        """Gets scan pattern configuration settings"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_set(
        self,
        *,
        name: str = "",
        scan_pattern: "_config__.ScanPattern" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Sets scan pattern cofiguration settings"""

        request = ScanPatternSetRequest()
        request.name = name
        if scan_pattern is not None:
            request.scan_pattern = scan_pattern

        return await self._unary_unary(
            "/blickfeld.system.services.ScanPattern/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        name: str = "",
        scan_pattern: "_config__.ScanPattern" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Sets scan pattern cofiguration settings"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                name=name,
                scan_pattern=scan_pattern,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "ScanPatternListResponse":
        """Return list of named scan patterns"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.system.services.ScanPattern/List",
            request,
            ScanPatternListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "ScanPatternListResponse":
        """Return list of named scan patterns"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_store(
        self,
        *,
        name: str = "",
        scan_pattern: "_config__.ScanPattern" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store scan pattern"""

        request = ScanPatternStoreRequest()
        request.name = name
        if scan_pattern is not None:
            request.scan_pattern = scan_pattern

        return await self._unary_unary(
            "/blickfeld.system.services.ScanPattern/Store",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store(
        self,
        *,
        name: str = "",
        scan_pattern: "_config__.ScanPattern" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store scan pattern"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store(
                name=name,
                scan_pattern=scan_pattern,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_delete(
        self,
        *,
        name: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete stored scan pattern"""

        request = ScanPatternDeleteRequest()
        request.name = name

        return await self._unary_unary(
            "/blickfeld.system.services.ScanPattern/Delete",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def delete(
        self,
        *,
        name: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete stored scan pattern"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_delete(
                name=name,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/system/data/firmware.proto, blickfeld/system/data/network_status.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__
from .. import config as _config__


@dataclass(eq=False, repr=False)
class Firmware(betterproto.Message):
    """
    The Firmware data message contains all relevant information about the
    current firmware, uploaded firmware and the status of an firmware
    installation.
    """

    pass


@dataclass(eq=False, repr=False)
class FirmwareInfo(betterproto.Message):
    """
    contains information about rauc bundle parsed from the associated manifest
    file
    """

    label: str = betterproto.string_field(1)
    """Human-readable label / version of the firmware"""

    version: "__base_data__.Version" = betterproto.message_field(2)
    """Version"""

    allowed_downgrade_version: "__base_data__.Version" = betterproto.message_field(3)
    """
    Lowest allowed version which can be downgraded to if this firmware is
    installed
    """


@dataclass(eq=False, repr=False)
class FirmwareStatus(betterproto.Message):
    """contains information about rauc bridge status"""

    idle: "FirmwareStatusIdle" = betterproto.message_field(1, group="status")
    """no bundle is available and no installation is happenning"""

    ready_to_install: "FirmwareStatusReadyToInstall" = betterproto.message_field(
        2, group="status"
    )
    """bundle is verified and available for installation"""

    installation: "FirmwareStatusInstallation" = betterproto.message_field(
        3, group="status"
    )
    """the installation process is ongoing"""

    failure: "FirmwareStatusFailure" = betterproto.message_field(4, group="status")
    """installation failed"""

    reboot: "FirmwareStatusReboot" = betterproto.message_field(5, group="status")
    """reboot has been triggered after successfull installation"""

    installed_firmware_info: "FirmwareInfo" = betterproto.message_field(6)
    """Currently installed firmware"""


@dataclass(eq=False, repr=False)
class FirmwareStatusIdle(betterproto.Message):
    """
    initial state when no bundle is available and no installation is happenning
    """

    pass


@dataclass(eq=False, repr=False)
class FirmwareStatusReadyToInstall(betterproto.Message):
    """state when bundle is verified and available for installation"""

    uploaded_firmware_info: "FirmwareInfo" = betterproto.message_field(1)
    """Firmware info of bundle which has been uploaded"""


@dataclass(eq=False, repr=False)
class FirmwareStatusInstallation(betterproto.Message):
    """state when the installation process is ongoing"""

    progress: "FirmwareStatusInstallationProgress" = betterproto.message_field(1)
    """Current installation progress"""


@dataclass(eq=False, repr=False)
class FirmwareStatusInstallationProgress(betterproto.Message):
    """
    Contains information about current state of installation procedure inside
    the tree of progress
    """

    percent: int = betterproto.uint32_field(1)
    """progress of installation in percents (0%-100%)"""

    message: str = betterproto.string_field(2)
    """log message describing the current installation step"""


@dataclass(eq=False, repr=False)
class FirmwareStatusFailure(betterproto.Message):
    """state when installation failed"""

    reason: str = betterproto.string_field(1)
    """Human-readable reason for installation failure"""


@dataclass(eq=False, repr=False)
class FirmwareStatusReboot(betterproto.Message):
    """
    state when reboot of the device has been triggered after successfull
    installation
    """

    pass


@dataclass(eq=False, repr=False)
class NetworkStatus(betterproto.Message):
    """
    Carries information about active connection: current statistics and network
    configuration
    """

    active_config: "_config__.Network" = betterproto.message_field(1)
    """Brings configuration information about currently active connection"""

    statistics: "NetworkStatusStatistics" = betterproto.message_field(2)
    """Brings statistical information about currently active connection"""


@dataclass(eq=False, repr=False)
class NetworkStatusStatistics(betterproto.Message):
    """
    (WIP) Carries statistical information about currently active connection
    """

    speed: str = betterproto.string_field(1)
    """Link speed"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/core_processing/services/acceleration.proto, blickfeld/core_processing/services/health.proto, blickfeld/core_processing/services/point_cloud.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class AccelerationStreamResponse(betterproto.Message):
    """Stream response to acceleration stream request"""

    buffer: "_data__.AccelerationBuffer" = betterproto.message_field(1)
    """Buffer with acceleration"""


@dataclass(eq=False, repr=False)
class AccelerationGetFilteredResponse(betterproto.Message):
    """Response to acceleration get filtered request"""

    acceleration: "_data__.Acceleration" = betterproto.message_field(1)
    """Single filtered acceleration"""


@dataclass(eq=False, repr=False)
class PointCloudStreamResponse(betterproto.Message):
    """Stream response to point cloud stream request"""

    frame: "_data__.Frame" = betterproto.message_field(1)
    """Point cloud frame"""


@dataclass(eq=False, repr=False)
class PointCloudGetResponse(betterproto.Message):
    """Response to point cloud get request"""

    frame: "_data__.Frame" = betterproto.message_field(1)
    """Point cloud frame"""


@dataclass(eq=False, repr=False)
class PointCloudEnableDemoRequest(betterproto.Message):
    """Response to point cloud enable demo request"""

    recording: "_config__.DemoRecording" = betterproto.enum_field(1)
    """Select demo recording"""


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """Response to health get request"""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """Stream response to health watch request"""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


class Acceleration(betterproto.ServiceStub):
    """
    The Acceleration Service provides access to the accelerometer of the
    device.
    """

    async def async_stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["AccelerationStreamResponse"]:
        """
        Streams raw data from accelerometer [WARNING] Raw data is acquired with
        a high sampling frequency and sent out in larger chunks. To estimate
        the static state of the device the GetFiltered method should be used.
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.core_processing.services.Acceleration/Stream",
            request,
            AccelerationStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["AccelerationStreamResponse"]:
        """
        Streams raw data from accelerometer [WARNING] Raw data is acquired with
        a high sampling frequency and sent out in larger chunks. To estimate
        the static state of the device the GetFiltered method should be used.
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get_filtered(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "AccelerationGetFilteredResponse":
        """Get filtered sample from accelerometer"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.core_processing.services.Acceleration/GetFiltered",
            request,
            AccelerationGetFilteredResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_filtered(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "AccelerationGetFilteredResponse":
        """Get filtered sample from accelerometer"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_filtered(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class PointCloud(betterproto.ServiceStub):
    """
    Point Cloud service for core point cloud stream. It is the direct result of
    the internal processing pipeline. The output is not post-processed further.
    The post-processed output can be retrieved from the percept-processing
    PointCloud service.
    """

    async def async_stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["PointCloudStreamResponse"]:
        """
        Stream a point cloud stream NOTE: This activates the sensor if it is
        currently idle.
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.core_processing.services.PointCloud/Stream",
            request,
            PointCloudStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["PointCloudStreamResponse"]:
        """
        Stream a point cloud stream NOTE: This activates the sensor if it is
        currently idle.
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PointCloudGetResponse":
        """
        Get a single point cloud NOTE: This activates the sensor if it is
        currently idle.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.core_processing.services.PointCloud/Get",
            request,
            PointCloudGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PointCloudGetResponse":
        """
        Get a single point cloud NOTE: This activates the sensor if it is
        currently idle.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_enable_demo(
        self,
        *,
        recording: "_config__.DemoRecording" = 0,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Enable demo data set It will be re-played on Stream requests"""

        request = PointCloudEnableDemoRequest()
        request.recording = recording

        return await self._unary_unary(
            "/blickfeld.core_processing.services.PointCloud/EnableDemo",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def enable_demo(
        self,
        *,
        recording: "_config__.DemoRecording" = 0,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Enable demo data set It will be re-played on Stream requests"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_enable_demo(
                recording=recording,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Health(betterproto.ServiceStub):
    """
    The health service provides methods to monitor operational status of the
    core_processing module
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the core_processing module"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.core_processing.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the core_processing module"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        Can be used to attach to the health monitoring status information of
        the core_processing module
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.core_processing.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        Can be used to attach to the health monitoring status information of
        the core_processing module
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/core_processing/data/acceleration.proto, blickfeld/core_processing/data/acceleration_buffer.proto, blickfeld/core_processing/data/frame.proto, blickfeld/core_processing/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import numpy as np

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Acceleration(betterproto.Message):
    """A measurement of the Accelerometer"""

    x: float = betterproto.float_field(1)
    """X component of acceleration vector"""

    y: float = betterproto.float_field(2)
    """Y component of acceleration vector"""

    z: float = betterproto.float_field(3)
    """Z component of acceleration vector"""


@dataclass(eq=False, repr=False)
class AccelerationBuffer(betterproto.Message):
    """Buffer for multiple Acceleration samples"""

    timestamp: int = betterproto.uint64_field(1)
    """Timestamp of the first sample in the buffer."""

    sampling_period: int = betterproto.uint64_field(2)
    """Sampling period of the acceleration samples"""

    cartesian: bytes = betterproto.bytes_field(3)
    """
    Cartesian 3-dimensional array in row-major format with [x, y, z] tuples.
    Type: Float32
    """


@dataclass(eq=False, repr=False)
class Frame(betterproto.Message):
    """This section describes the contents of a point cloud frame."""

    id: int = betterproto.uint64_field(1)
    """Incremental frame ID since startup of the device."""

    timestamp: int = betterproto.uint64_field(2)
    """
    Timestamp of start of frame. Timestamp is synchronized if Time
    Synchronization is configured.
    """

    binary: "FrameBinary" = betterproto.message_field(3)
    """Binary buffer with point cloud vectors"""


@dataclass(eq=False, repr=False)
class FrameBinary(betterproto.Message):
    """
    The fields of a point cloud frame are encoded as binary arrays. Binary
    types instead of native Protobuf types are used to improve the performance.
    NOTE: All byte arrays are encoded in Little Endian.
    """

    length: int = betterproto.uint32_field(1)
    """Number of entries in arrays below."""

    cartesian: np.ndarray = betterproto.bytes_field(2, numpy_dtype="(3,)<f4")
    """
    Cartesian coordinates. 3-dimensional array in row-major format with [x, y,
    z] tuples. Type: Float32
    """

    photon_count: np.ndarray = betterproto.bytes_field(3, numpy_dtype="<u2")
    """Estimated number of photons. 1-dimensional array. Type: UInt16"""

    direction_id: np.ndarray = betterproto.bytes_field(5, numpy_dtype="<u4")
    """
    Unique direction identifiers within a frame and scan pattern. 1-dimensional
    array. Type: UInt32
    """

    timestamp: np.ndarray = betterproto.bytes_field(6, numpy_dtype="<u8")
    """Unix timestamp of points. Type: UInt64"""


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Message representing health"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is STATE_OK."""

    channel_status: "HealthChannelStatus" = betterproto.message_field(3)
    """Status of internal channels"""

    server_status: "HealthServerStatus" = betterproto.message_field(4)
    """Status of gRPC server"""

    error_flags: "HealthErrorFlags" = betterproto.message_field(5)
    """Error flags which give insight in fine-grained errors"""


@dataclass(eq=False, repr=False)
class HealthChannelStatus(betterproto.Message):
    """Status for typed channels producing raw-data"""

    point_cloud: "HealthChannelStatusPointCloud" = betterproto.message_field(1)
    """Point Cloud status"""

    acceleration: "HealthChannelStatusAcceleration" = betterproto.message_field(2)
    """Acceleration status"""

    adc: "HealthChannelStatusAdc" = betterproto.message_field(3)
    """Adc status"""

    vertical_beam_deflection_control: "HealthChannelStatusBeamDeflectionControl" = (
        betterproto.message_field(4)
    )
    """Vertical Beam Deflection Control status"""

    horizontal_beam_deflection_control: "HealthChannelStatusBeamDeflectionControl" = (
        betterproto.message_field(5)
    )
    """Horizontal Beam Deflection Control status"""

    magnetic_field: "HealthChannelStatusMagneticField" = betterproto.message_field(6)
    """Magnetic Field status"""


@dataclass(eq=False, repr=False)
class HealthChannelStatusPointCloud(betterproto.Message):
    """Status of Point Cloud channel"""

    point_rate: float = betterproto.float_field(1)
    """Processed points per second"""

    scanline_rate: float = betterproto.float_field(2)
    """Scanlines per second"""

    frame_rate: float = betterproto.float_field(3)
    """Frames per second"""

    data_rate: float = betterproto.float_field(4)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusAcceleration(betterproto.Message):
    """Status of Acceleration channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Processed buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusAdc(betterproto.Message):
    """Status of Adc channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Processed buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusBeamDeflectionControl(betterproto.Message):
    """Status of BeamDeflectionControl channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Produced buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusMagneticField(betterproto.Message):
    """Status of Magnetic-Field channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Produced buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatus(betterproto.Message):
    """Status for Server / Client activities"""

    point_cloud: "HealthServerStatusPointCloud" = betterproto.message_field(1)
    """Point Cloud service"""

    acceleration: "HealthServerStatusAcceleration" = betterproto.message_field(2)
    """Acceleration service"""

    adc: "HealthServerStatusAdc" = betterproto.message_field(3)
    """Adc service"""

    beam_deflection_control: "HealthServerStatusBeamDeflectionControl" = (
        betterproto.message_field(4)
    )
    """Beam Deflection Control service"""

    health: "HealthServerStatusHealth" = betterproto.message_field(5)
    """Health service"""

    magnetic_field: "HealthServerStatusMagneticField" = betterproto.message_field(6)
    """Magnetic Field service"""


@dataclass(eq=False, repr=False)
class HealthServerStatusPointCloud(betterproto.Message):
    """Status for Point Cloud service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusAcceleration(betterproto.Message):
    """Status for Acceleration service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusAdc(betterproto.Message):
    """Status for Adc service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusBeamDeflectionControl(betterproto.Message):
    """Status for BeamDeflectionControl service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusHealth(betterproto.Message):
    """Status for Health service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusMagneticField(betterproto.Message):
    """Status for Magnetic Field service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthErrorFlags(betterproto.Message):
    """Error flags of internal data handling mechanisms"""

    packet_counter_mismatch_packager: bool = betterproto.bool_field(1)
    """TLP counter mismatch for Packager DMA"""

    packet_counter_mismatch_debug: bool = betterproto.bool_field(2)
    """TLP counter mismatch for Debug DMA"""

    exhausted_scanline_buffers: bool = betterproto.bool_field(3)
    """Dropping data because scanline buffers are exhausted"""

    exhausted_acceleration_buffers: bool = betterproto.bool_field(4)
    """Dropping data because acceleration buffers are exhausted"""

    exhausted_adc_buffers: bool = betterproto.bool_field(5)
    """Dropping data because adc buffers are exhausted"""

    exhausted_vertical_beam_deflection_control_buffers: bool = betterproto.bool_field(6)
    """
    Dropping data because vertical beam deflection control buffers are
    exhausted
    """

    exhausted_horizontal_beam_deflection_control_buffers: bool = betterproto.bool_field(
        7
    )
    """
    Dropping data because horizontal beam deflection control buffers are
    exhausted
    """

    exhausted_magnetic_field_buffers: bool = betterproto.bool_field(8)
    """Dropping data because magnetic field buffers are exhausted"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_pipeline/data/coordinate_system.proto, blickfeld/percept_pipeline/data/health.proto, blickfeld/percept_pipeline/data/point_cloud_type.proto, blickfeld/percept_pipeline/data/state.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


class State(betterproto.Enum):
    """The states of pipeline"""

    STATE_UNSPECIFIED = 0
    """State field is not set"""

    STATE_STARTING = 1
    """Pipeline is getting started currently."""

    STATE_STABLE = 2
    """
    Pipeline is stable input and output of the pipeline are achieving the same
    frequency. Only very few frames are allowed to be dropped.
    """

    STATE_UNSTABLE = 3
    """
    Pipeline is unstable if the pipeline cannot process all frames of the data
    source and thus has to drop frames to keep up.
    """

    STATE_BROKEN = 4
    """
    Pipeline is broken if the pipeline does not generate the expected outputs,
    i.e. frames or state list at all. This can happen if the devices in the
    'data source' cannot be reached.
    """

    STATE_RESTARTING = 5
    """
    Pipeline is restarting if there is a larger configuration change, i.e. data
    source changed.
    """

    STATE_IDLE = 6
    """No processing is happening in the pipeline."""


class PointCloudType(betterproto.Enum):
    """The type of the point cloud"""

    POINT_CLOUD_TYPE_UNSPECIFIED = 0
    """The zero value that is not accepted"""

    POINT_CLOUD_TYPE_FULL = 1
    """
    Full point cloud (combined point cloud of all devices or unfiltered point
    cloud coming from one device)
    """

    POINT_CLOUD_TYPE_FOREGROUND = 2
    """Point cloud containing only foreground/points from moving objects"""


class CoordinateSystem(betterproto.Enum):
    """Specifies the coordinates system the data is expressed in"""

    COORDINATE_SYSTEM_UNSPECIFIED = 0
    """
    Coordinate system not specified. Not accepted neither within a request nor
    as response
    """

    COORDINATE_SYSTEM_LOCAL = 1
    """
    The data is expressed in the coordiante system of the data-source/device
    """

    COORDINATE_SYSTEM_TRANSFORMED = 2
    """The data is expressed in a map coordinate system"""


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    A health message that contains information about the pipeline status and
    the module itself.
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module."""

    state_reason: str = betterproto.string_field(2)
    """Reason for the given state. It is not set if state is STATE_OK."""

    pipeline_state: "State" = betterproto.enum_field(3)
    """
    Current state of the pipeline. It indicates if the pipeline is stable,
    unstable or broken.
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/core_processing/config/demo_recording.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class DemoRecording(betterproto.Enum):
    """
    Demo recording which can be replayed instead of the real sensor data. This
    is also used for virtual devices which have no physical sensing unit.
    WARNING: The recordings are still preliminary. The demo test data concept
    will be refactored soon.
    """

    DEMO_RECORDING_UNSPECIFIED = 0
    """No demo recoding is specified"""

    DEMO_RECORDING_HIGH_RESOLUTION = 1
    """High resolution recording in the office with 200 + 30 scanlines"""

    DEMO_RECORDING_HIGH_FRAME_RATE = 2
    """High resolution recording in the office with 30 + 30 scanlines"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/laser/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is OK."""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/base/data/health.proto, blickfeld/base/data/version.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class HealthState(betterproto.Enum):
    """
    High-level state which indicates the current health state of the module. If
    the State is not OK, please refer to the details fields in the Health
    message to trace the root cause.
    """

    STATE_UNSPECIFIED = 0
    """The current state is unknown."""

    STATE_OK = 1
    """Everything is perfect."""

    STATE_WARNING = 2
    """
    There are non-critical errors. This might result in unexpected error and
    should be investigated for production set-ups.
    """

    STATE_FAILED = 3
    """The module is in a failed state and can no longer operate."""

    STATE_BUSY = 4
    """The module is currently busy. Requests might get aborted or delayed."""


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    The Health message and corresponding services are used to indicate their
    health & status. They are available on different layers. Upstream / higher
    layers usually aggregate the Health of their child modules. In the end, the
    goal is, to get an aggregated product Health which reliably indicates if
    everything is functional.
    """

    pass


@dataclass(eq=False, repr=False)
class Version(betterproto.Message):
    """
    General message which is used for version information.
    https://semver.org[Semantic versioning] is used to identify a software
    state. It additional adds the short hash of the corresponding commit.
    """

    major: int = betterproto.uint32_field(1)
    """Increments when we make incompatible API changes."""

    minor: int = betterproto.uint32_field(2)
    """
    Increments when we add functionality in a backwards compatible manner.
    """

    patch: int = betterproto.uint32_field(3)
    """Increments when we make backwards compatible bug fixes."""

    revision: str = betterproto.string_field(4)
    """Short hash of the commit in the corresponding project repo."""

    is_release: bool = betterproto.bool_field(5)
    """
    This flag indicates a release. An official release is fully tested &
    reviewed.
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/beam_deflection_control/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    Health message containing information about operation, status and error of
    a beam deflection controller.
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is OK."""


@dataclass(eq=False, repr=False)
class HealthStats(betterproto.Message):
    """
    This message defines the stats that can be retrieved from a beam deflection
    controller.
    """

    time: float = betterproto.float_field(1)
    """Current time in s."""

    bb_frequency: float = betterproto.float_field(2)
    """Current BB frequency estimation (only set when running BB)."""

    max_mechanical_scan_range: float = betterproto.float_field(3)
    """
    Maximum MSR which occured. Observer q1 in case of CL, sensor atan in case
    of BB.
    """

    min_dac_u0: int = betterproto.uint32_field(4)
    """Minimum DAC value wich occured for piezo 0."""

    max_dac_u0: int = betterproto.uint32_field(5)
    """Maximum DAC value wich occured for piezo 0."""

    min_dac_u1: int = betterproto.uint32_field(6)
    """Minimum DAC value wich occured for piezo 1."""

    max_dac_u1: int = betterproto.uint32_field(7)
    """Maximum DAC value wich occured for piezo 1."""

    max_cycles: int = betterproto.uint32_field(8)
    """Maximum amount of cycles needed for control loop."""
